@@ -33,7 +33,7 @@ public final class Capability implements Serializable {
     }
 
     public Capability() {
-        this(MASK_MULTI_CLASSLOADER|MASK_PIPE_THROTTLING);
+        this(MASK_MULTI_CLASSLOADER|MASK_PIPE_THROTTLING|MASK_MIMIC_EXCEPTION);
     }
 
     /**
@@ -55,6 +55,10 @@ public final class Capability implements Serializable {
         return (mask& MASK_PIPE_THROTTLING)!=0;
     }
 
+    public boolean hasMimicException() {
+        return (mask&MASK_MIMIC_EXCEPTION)!=0;
+    }
+
     /**
      * Writes out the capacity preamble.
      */
@@ -103,6 +107,11 @@ public final class Capability implements Serializable {
      */
     private static final long MASK_PIPE_THROTTLING = 4L;
 
+    /**
+     * Supports {@link MimicException}.
+     */
+    private static final long MASK_MIMIC_EXCEPTION = 8L;
+
     static final byte[] PREAMBLE;
 
     public static final Capability NONE = new Capability(0);
