@@ -137,7 +137,7 @@ public class Launcher {
     public File tcpPortFile=null;
 
 
-    @Option(name="-auth",metaVar="user:pass",usage="If your Hudson is security-enabeld, specify a valid user name and password.")
+    @Option(name="-auth",metaVar="user:pass",usage="If your Hudson is security-enabled, specify a valid user name and password.")
     public String auth = null;
 
     public InetSocketAddress connectionTarget = null;
@@ -160,7 +160,7 @@ public class Launcher {
      */
     @Option(name="-noCertificateCheck")
     public void setNoCertificateCheck(boolean _) throws NoSuchAlgorithmException, KeyManagementException {
-        System.out.println("Skipping HTTPS certificate checks altoghether. Note that this is not secure at all.");
+        System.out.println("Skipping HTTPS certificate checks altogether. Note that this is not secure at all.");
         SSLContext context = SSLContext.getInstance("TLS");
         context.init(null, new TrustManager[]{new NoCheckTrustManager()}, new java.security.SecureRandom());
         HttpsURLConnection.setDefaultSSLSocketFactory(context.getSocketFactory());
@@ -227,14 +227,14 @@ public class Launcher {
      * Parses the connection arguments from JNLP file given in the URL.
      */
     public List<String> parseJnlpArguments() throws ParserConfigurationException, SAXException, IOException, InterruptedException {
+        if (secret != null) {
+            slaveJnlpURL = new URL(slaveJnlpURL + "?encrypt=true");
+            if (slaveJnlpCredentials != null) {
+                throw new IOException("-jnlpCredentials and -secret are mutually exclusive");
+            }
+        }
         while (true) {
             try {
-                if (secret != null) {
-                    slaveJnlpURL = new URL(slaveJnlpURL + "?encrypt=true");
-                    if (slaveJnlpCredentials != null) {
-                        throw new IOException("-jnlpCredentials and -secret are mutually exclusive");
-                    }
-                }
                 URLConnection con = slaveJnlpURL.openConnection();
                 if (con instanceof HttpURLConnection && slaveJnlpCredentials != null) {
                     HttpURLConnection http = (HttpURLConnection) con;
@@ -343,7 +343,7 @@ public class Launcher {
      */
     private void runAsTcpServer() throws IOException, InterruptedException {
         // if no one connects for too long, assume something went wrong
-        // and avoid hanging foreever
+        // and avoid hanging forever
         ServerSocket ss = new ServerSocket(0,1);
         ss.setSoTimeout(30*1000);
 
@@ -381,7 +381,7 @@ public class Launcher {
      */
     private void runAsTcpClient() throws IOException, InterruptedException {
         // if no one connects for too long, assume something went wrong
-        // and avoid hanging foreever
+        // and avoid hanging forever
         Socket s = new Socket(connectionTarget.getAddress(),connectionTarget.getPort());
 
         runOnSocket(s);
@@ -424,6 +424,9 @@ public class Launcher {
         // and messing up the stream.
         OutputStream os = new StandardOutputStream();
         System.setOut(System.err);
+
+        // System.in/out appear to be already buffered (at least that was the case in Linux and Windows as of Java6)
+        // so we are not going to double-buffer these.
         main(System.in, os, mode, ping);
     }
 
