@@ -1,10 +1,15 @@
 package hudson.remoting;
 
+import hudson.remoting.ChannelRunner.Fork;
+import hudson.remoting.ChannelRunner.InProcessCompatibilityMode;
 import org.jvnet.hudson.test.Bug;
 
 import java.io.IOException;
 import java.io.ObjectInputStream;
 import java.io.Serializable;
+import java.net.URL;
+import java.net.URLClassLoader;
+import java.util.concurrent.TimeUnit;
 
 /**
  * @author Kohsuke Kawaguchi
@@ -100,7 +105,7 @@ public class ChannelTest extends RmiTestBase {
     public void testWaitForRemoteProperty() throws Exception {
         Future<Void> f = channel.callAsync(new WaitForRemotePropertyCallable());
         assertEquals("bar", channel.waitForRemoteProperty("foo"));
-        assertTrue(f.isDone());
+        f.get(1, TimeUnit.SECONDS);
     }
 
     private static class WaitForRemotePropertyCallable implements Callable<Void, Exception> {
@@ -145,4 +150,22 @@ public class ChannelTest extends RmiTestBase {
             return null;
         }
     }
+
+    public void testClassLoaderHolder() throws Exception {
+        URLClassLoader ucl = new URLClassLoader(new URL[0]);
+        ClassLoaderHolder h = channel.call(new Echo<ClassLoaderHolder>(new ClassLoaderHolder(ucl)));
+        assertSame(ucl,h.get());
+    }
+
+    private static class Echo<T> implements Callable<T,RuntimeException> {
+        private final T t;
+
+        Echo(T t) {
+            this.t = t;
+        }
+
+        public T call() throws RuntimeException {
+            return t;
+        }
+    }
 }
